"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = void 0;

var _changeCase = require("change-case");

var _chalk = _interopRequireDefault(require("chalk"));

var fs = _interopRequireWildcard(require("fs-extra"));

var path = _interopRequireWildcard(require("path"));

var logger = _interopRequireWildcard(require("../utils/logger"));

var _getModularRoot = _interopRequireDefault(require("../utils/getModularRoot"));

var _actionPreflightCheck = _interopRequireDefault(require("../utils/actionPreflightCheck"));

var _isModularType = _interopRequireDefault(require("../utils/isModularType"));

var _execAsync = _interopRequireDefault(require("../utils/execAsync"));

var _getLocation = _interopRequireDefault(require("../utils/getLocation"));

var _setupEnv = require("../utils/setupEnv");

var _createPaths = _interopRequireDefault(require("../utils/createPaths"));

var _printHostingInstructions = _interopRequireDefault(require("./printHostingInstructions"));

var _fileSizeReporter = require("./fileSizeReporter");

var _checkBrowsers = require("../utils/checkBrowsers");

var _checkRequiredFiles = _interopRequireDefault(require("../utils/checkRequiredFiles"));

var _createEsbuildBrowserslistTarget = _interopRequireDefault(require("../utils/createEsbuildBrowserslistTarget"));

var _webpackFileSizeReporter = require("./webpackFileSizeReporter");

var _esbuildFileSizeReporter = require("./esbuildFileSizeReporter");

var _getPackageDependencies = require("../utils/getPackageDependencies");

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _getRequireWildcardCache(nodeInterop) { if (typeof WeakMap !== "function") return null; var cacheBabelInterop = new WeakMap(); var cacheNodeInterop = new WeakMap(); return (_getRequireWildcardCache = function (nodeInterop) { return nodeInterop ? cacheNodeInterop : cacheBabelInterop; })(nodeInterop); }

function _interopRequireWildcard(obj, nodeInterop) { if (!nodeInterop && obj && obj.__esModule) { return obj; } if (obj === null || typeof obj !== "object" && typeof obj !== "function") { return { default: obj }; } var cache = _getRequireWildcardCache(nodeInterop); if (cache && cache.has(obj)) { return cache.get(obj); } var newObj = {}; var hasPropertyDescriptor = Object.defineProperty && Object.getOwnPropertyDescriptor; for (var key in obj) { if (key !== "default" && Object.prototype.hasOwnProperty.call(obj, key)) { var desc = hasPropertyDescriptor ? Object.getOwnPropertyDescriptor(obj, key) : null; if (desc && (desc.get || desc.set)) { Object.defineProperty(newObj, key, desc); } else { newObj[key] = obj[key]; } } } newObj.default = obj; if (cache) { cache.set(obj, newObj); } return newObj; }

async function buildApp(target) {
  // True if there's no preference set - or the preference is for webpack.
  const useWebpack = !process.env.USE_MODULAR_WEBPACK || process.env.USE_MODULAR_WEBPACK === 'true'; // True if the preferene IS set and the preference is esbuid.

  const useEsbuild = process.env.USE_MODULAR_ESBUILD && process.env.USE_MODULAR_ESBUILD === 'true'; // If you want to use webpack then we'll always use webpack. But if you've indicated
  // you want esbuild - then we'll switch you to the new fancy world.

  const isEsbuild = !useWebpack || useEsbuild; // Setup Paths

  const modularRoot = (0, _getModularRoot.default)();
  const targetDirectory = await (0, _getLocation.default)(target);
  const targetName = (0, _changeCase.paramCase)(target);
  const paths = await (0, _createPaths.default)(target);
  await (0, _checkBrowsers.checkBrowsers)(targetDirectory);
  let previousFileSizes;

  if (isEsbuild) {
    previousFileSizes = await (0, _esbuildFileSizeReporter.esbuildMeasureFileSizesBeforeBuild)(paths.appBuild);
  } else {
    previousFileSizes = await (0, _webpackFileSizeReporter.webpackMeasureFileSizesBeforeBuild)(paths.appBuild);
  } // Warn and crash if required files are missing


  await (0, _checkRequiredFiles.default)([paths.appHtml, paths.appIndexJs]);
  logger.log('Creating an optimized production build...');
  await fs.emptyDir(paths.appBuild);
  await fs.copy(paths.appPublic, paths.appBuild, {
    dereference: true,
    filter: file => file !== paths.appHtml,
    overwrite: true
  });
  let assets;

  if (isEsbuild) {
    const {
      default: buildEsbuildApp
    } = await Promise.resolve().then(() => _interopRequireWildcard(require('../esbuild-scripts/build')));
    const result = await buildEsbuildApp(target, paths);
    assets = (0, _esbuildFileSizeReporter.createEsbuildAssets)(paths, result);
  } else {
    // create-react-app doesn't support plain module outputs yet,
    // so --preserve-modules has no effect here
    const buildScript = require.resolve('modular-scripts/react-scripts/scripts/build.js');

    const browserTarget = (0, _createEsbuildBrowserslistTarget.default)(targetDirectory); // TODO: this shouldn't be sync

    await (0, _execAsync.default)('node', [buildScript], {
      cwd: targetDirectory,
      log: false,
      // @ts-ignore
      env: {
        ESBUILD_TARGET_FACTORY: JSON.stringify(browserTarget),
        MODULAR_ROOT: modularRoot,
        MODULAR_PACKAGE: target,
        MODULAR_PACKAGE_NAME: targetName
      }
    });
    const statsFilePath = path.join(paths.appBuild, 'bundle-stats.json');

    try {
      var _stats$warnings;

      const stats = await fs.readJson(statsFilePath);

      if (stats !== null && stats !== void 0 && (_stats$warnings = stats.warnings) !== null && _stats$warnings !== void 0 && _stats$warnings.length) {
        logger.log(_chalk.default.yellow('Compiled with warnings.\n'));
        logger.log(stats.warnings.join('\n\n'));
        logger.log('\nSearch for the ' + _chalk.default.underline(_chalk.default.yellow('keywords')) + ' to learn more about each warning.');
      } else {
        logger.log(_chalk.default.green('Compiled successfully.\n'));
      }

      assets = (0, _webpackFileSizeReporter.createWebpackAssets)(paths, stats);
    } finally {
      await fs.remove(statsFilePath);
    }
  } // Add dependencies from source and bundled dependencies to target package.json


  const packageDependencies = await (0, _getPackageDependencies.getPackageDependencies)(target);
  const targetPackageJson = await fs.readJSON(path.join(targetDirectory, 'package.json'));
  targetPackageJson.dependencies = packageDependencies;
  targetPackageJson.bundledDependencies = Object.keys(packageDependencies); // Copy selected fields of package.json over

  await fs.writeJSON(path.join(paths.appBuild, 'package.json'), {
    name: targetPackageJson.name,
    version: targetPackageJson.version,
    license: targetPackageJson.license,
    modular: targetPackageJson.modular,
    dependencies: targetPackageJson.dependencies
  }, {
    spaces: 2
  });
  (0, _fileSizeReporter.printFileSizesAfterBuild)(assets, previousFileSizes);
  (0, _printHostingInstructions.default)(target, paths.publicUrlOrPath, paths.publicUrlOrPath, paths.appBuild);
}

async function build(target, preserveModules = true, includePrivate = false) {
  const targetDirectory = await (0, _getLocation.default)(target);
  await (0, _setupEnv.setupEnvForDirectory)(targetDirectory);

  if ((0, _isModularType.default)(targetDirectory, 'app')) {
    await buildApp(target);
  } else {
    const {
      buildPackage
    } = await Promise.resolve().then(() => _interopRequireWildcard(require('./buildPackage'))); // ^ we do a dynamic import here to defer the module's initial side effects
    // till when it's actually needed (i.e. now)

    await buildPackage(target, preserveModules, includePrivate);
  }
}

var _default = (0, _actionPreflightCheck.default)(build);

exports.default = _default;
//# sourceMappingURL=index.js.map