"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.convert = convert;

var _execa = _interopRequireDefault(require("execa"));

var fs = _interopRequireWildcard(require("fs-extra"));

var path = _interopRequireWildcard(require("path"));

var _rimraf = _interopRequireDefault(require("rimraf"));

var _changeCase = require("change-case");

var _isModularType = require("./utils/isModularType");

var logger = _interopRequireWildcard(require("./utils/logger"));

var _check = require("./check");

var _gitActions = require("./utils/gitActions");

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _getRequireWildcardCache(nodeInterop) { if (typeof WeakMap !== "function") return null; var cacheBabelInterop = new WeakMap(); var cacheNodeInterop = new WeakMap(); return (_getRequireWildcardCache = function (nodeInterop) { return nodeInterop ? cacheNodeInterop : cacheBabelInterop; })(nodeInterop); }

function _interopRequireWildcard(obj, nodeInterop) { if (!nodeInterop && obj && obj.__esModule) { return obj; } if (obj === null || typeof obj !== "object" && typeof obj !== "function") { return { default: obj }; } var cache = _getRequireWildcardCache(nodeInterop); if (cache && cache.has(obj)) { return cache.get(obj); } var newObj = {}; var hasPropertyDescriptor = Object.defineProperty && Object.getOwnPropertyDescriptor; for (var key in obj) { if (key !== "default" && Object.prototype.hasOwnProperty.call(obj, key)) { var desc = hasPropertyDescriptor ? Object.getOwnPropertyDescriptor(obj, key) : null; if (desc && (desc.get || desc.set)) { Object.defineProperty(newObj, key, desc); } else { newObj[key] = obj[key]; } } } newObj.default = obj; if (cache) { cache.set(obj, newObj); } return newObj; }

process.on('SIGINT', () => {
  (0, _gitActions.stashChanges)();
  process.exit(1);
});

async function convert(cwd = process.cwd()) {
  if (!(0, _gitActions.cleanGit)(cwd)) {
    throw new Error('You have unsaved changes. Please save or stash them before we attempt to convert this react app to modular app.');
  }

  try {
    if (!(0, _isModularType.isValidModularRootPackageJson)(cwd) || !fs.existsSync(path.join(cwd, 'packages'))) {
      const {
        initModularFolder
      } = await Promise.resolve().then(() => _interopRequireWildcard(require('./init')));
      logger.debug("You don't have a modular repo initialized. Setting it up for you now.");
      await initModularFolder(cwd, true);
    }

    const rootPackageJson = await fs.readJson(path.join(cwd, 'package.json'));
    const packageName = rootPackageJson.name;
    logger.log(`Setting up new modular app for ${packageName}`); // Create a modular app package folder

    const packageTypePath = path.join(__dirname, '../types', 'app');
    const newPackagePath = path.join(cwd, 'packages', (0, _changeCase.paramCase)(packageName));
    fs.mkdirpSync(newPackagePath);
    const newPackageJson = fs.readJsonSync(path.join(packageTypePath, 'packagejson')); // Bring key props from root package.json to new app

    newPackageJson.name = packageName;
    fs.writeJsonSync(path.join(newPackagePath, 'package.json'), newPackageJson, {
      spaces: 2
    }); // Move the cwd folders to the modular app

    const srcFolders = ['src', 'public'];
    srcFolders.forEach(dir => {
      if (fs.existsSync(path.join(cwd, dir))) {
        fs.moveSync(path.join(cwd, dir), path.join(newPackagePath, dir));
      } else {
        fs.copySync(path.join(packageTypePath, dir), path.join(newPackagePath, dir), {
          overwrite: true
        });
      }
    });
    logger.debug('Set tsconfig.json to include your workspaces');
    let tsConfig = {
      extends: 'modular-scripts/tsconfig.json',
      include: ['modular', 'packages/**/src']
    };
    const rootTSConfigPath = path.join(cwd, 'tsconfig.json'); // If they have a tsconfig, include packages/**/src

    if (fs.existsSync(rootTSConfigPath)) {
      tsConfig = fs.readJsonSync(rootTSConfigPath); // The tsconfig might already have the necessary includes
      // but just in case, this will ensure that it does

      const include = tsConfig.include || [];
      tsConfig.include = include.filter(key => !['src', 'packages/**/src'].includes(key));
      tsConfig.include.push('packages/**/src');
    }

    fs.writeJsonSync(rootTSConfigPath, tsConfig, {
      spaces: 2
    });
    fs.writeJSONSync(path.join(newPackagePath, 'tsconfig.json'), {
      extends: path.relative(newPackagePath, cwd) + '/tsconfig.json'
    }, {
      spaces: 2
    });

    if (fs.existsSync(path.join(newPackagePath, 'src', 'react-app-env.d.ts'))) {
      logger.debug('Updating your react-app-env.d.ts for modular-scripts');
      fs.writeFileSync(path.join(newPackagePath, 'src', 'react-app-env.d.ts'), fs.readFileSync(path.join(newPackagePath, 'src', 'react-app-env.d.ts'), 'utf8').replace('<reference types="react-scripts" />', '<reference types="modular-scripts/react-app-env" />'));
    }

    logger.debug('Migrating your setupTests file to modular');
    const setupFileName = 'setupTests';
    const setupFilesExts = ['js', 'ts'];
    setupFilesExts.forEach(ext => {
      const file = `${setupFileName}.${ext}`;

      if (fs.existsSync(path.join(newPackagePath, 'src', file))) {
        fs.writeFileSync(path.join(cwd, 'modular', file), fs.readFileSync(path.join(newPackagePath, 'src', file), 'utf8'));

        _rimraf.default.sync(path.join(newPackagePath, 'src', file));
      }
    });
    logger.debug('Removing react-scripts from dependencies list');
    const rootDeps = rootPackageJson.dependencies || {};
    rootPackageJson.dependencies = Object.keys(rootDeps).reduce((acc, dep) => {
      if (dep !== 'react-scripts') {
        return { ...acc,
          [dep]: rootDeps[dep]
        };
      }

      return acc;
    }, {}); // Deps that need to be reintroduced because we removed react-scripts

    const additionalDeps = ['eslint-config-modular-app'];
    fs.writeJsonSync(path.join(cwd, 'package.json'), rootPackageJson, {
      spaces: 2
    });
    logger.log('Running yarn to update dependencies');

    _execa.default.sync('yarnpkg', ['--silent', 'add', '-W', ...additionalDeps], {
      cwd
    });

    logger.log('Validating your modular project...');
    await (0, _check.check)();
  } catch (err) {
    logger.error(err);
    (0, _gitActions.stashChanges)();
  }
}
//# sourceMappingURL=convert.js.map